class Style:
    focus = "Y"
    italic = "/"
    underline = "_"

    opt_quest = italic + "m"
    opt_task = italic + "c"
    text = ""
    cluster_key = "b"
    cluster_title = ""
    quest_key = italic + "b"
    tasks = "y"
    lcmd = 'r'
    cmd = ""
    code_key = ""
    skills = "c"
    play = "g"
    new = "g"

    nothing = "m"
    started = "r"
    required = "y"
    complete = "g"

    shell = "r" # extern shell cmds
    htext = ""
    check = "g"
    uncheck = "y"
    param = "c"

    bar_main = "M"
    bar_ext = "Y"
    bar_xp = "C"
    flags_true = "G"
    flags_false = "Y"
    bar_skills = "B"
    bar_cmds = "Y"
    bar_bg = "M"